#!/usr/bin/python3
import argparse
import time
import colorama
import os
import subprocess
import gettext
from ttyrecgenerator import RecSession
import pkg_resources
gettext.install('recpermissions', pkg_resources.resource_filename('recpermissions', 'locale'))

#We change permissions for the howto
os.system("mkdir -p directory")
os.system("touch directory/prueba.txt")
os.system("mkdir -p directory_empty")
os.system("chown -R apache:apache *")
os.system("chmod -R  777 *")

r=RecSession()
r.comment("# " + _("This is a video to show how to use 'recpermissions' command"))
r.comment("# " + _("We list files with permissions and owners"))
r.command("ls -la")

r.comment("# " + _("We want to change them to root:root owner. Files to 640 permissions, directories to 750. We want to remove empty directories too, so we use:"))
r.command("recpermissions --user root --group root --files 640 --directories 750 --remove_emptydirs " + os.getcwd())
r.comment("# " + _("We check the result"))
r.command("ls -la")
r.comment("# " + _("That's all"))
time.sleep(30)

#We remove example
os.system("rm -Rf directory")



## @namespace recpermissions.core
## @brief Core functions of the package
import platform
import sys

if platform.system()=="Windows":
    print("This script only works on Linux")
    sys.exit(0)

import argparse
import gettext
import grp
import locale
import os
import pkg_resources
import pwd
import shutil

from colorama import Fore, Style, init as colorama_init
from recpermissions.version import __versiondate__, __version__
from stat import ST_MODE

try:
    t=gettext.translation('recpermissions',pkg_resources.resource_filename("recpermissions","locale"))
    _=t.gettext
except:
    _=str

## Returns a localized int
## @param value Integer to localize
## @return string
def localized_int(value):
    return locale.format_string("%d", value, True)

## Check if a directory is empty
## @param dir String with the directory to check
## @return boolean
def is_dir_empty(dir):
    if not os.listdir(dir):
        return True
    else:
        return False

## Gets octal string permissions from a file
## @param path String with the path. Can be a dir or a file
## @return string "644" or "755", for example
def get_octal_string_permissions(path):
    return oct(os.stat(path)[ST_MODE])[-3:]

## Sets octal string permissions to a file
## @param path String with the path. Can be a dir or a file
## @param octal String with octal permissions. "644" or "755" for example
## @return Boolean if file has been changed
def set_octal_string_permissions(path, octal):
    if octal==None:
        return False
    if get_octal_string_permissions(path)==octal:
        return False
    else:
        try:
            os.chmod(path, int(octal, 8))
            return True
        except:
            return False

## Returns if the octal string has valid octal permissions
## @param octal String with octal permissions. "644" or "755" for example
## @return Boolean if the octal string is valid
def is_octal_string_permissions_valid(octal):
    if len(octal)==3 and octal.isdigit()==True and octal.find("9")==-1:
         return True
    return False

## Gets user and root from a path
## @param path String with the path. Can be a dir or a file
## @return a tuple (root, root), for example. If uid of the file isn't in /etc/passwd, returns uid and gid
def get_file_ownership(path):
    try:
        return (pwd.getpwuid(os.stat(path).st_uid).pw_name, grp.getgrgid(os.stat(path).st_gid).gr_name)
    except:
        return (os.stat(path).st_uid, os.stat(path).st_gid)

## Set file user and grup
## @param path String with the path. Can be a dir or a file
## @param user String or None. If none it doesn't change the user
## @param group String or None. If none it doesn't change the group
## @return Boolean if file has been changed
def set_file_ownership(path, user, group):
    if (user, group)==(None, None):
        return False
    tuple=get_file_ownership(path)
    if tuple==(user, group):
        return False
    else:
        user=tuple[0] if user==None else user
        group=tuple[1] if group==None else group
        try:
            shutil.chown(path, user, group)
            return True
        except:
            return False

## recpermissions main script
## If arguments is None, launches with sys.argc parameters. Entry point is recpermissions:main
## You can call with main(['--pretend']). It's equivalento to os.system('recpermissions --pretend')
## @param arguments is an array with parser arguments. For example: ['--max_files_to_store','9']. 
def main(arguments=None):
    parser=argparse.ArgumentParser(prog='recpermissions', description=_('Change Linux permissions and ownership in one step. It can delete empty directories when necessary.'), epilog=_("Developed by Mariano Muñoz 2018-{}".format(__versiondate__.year)), formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('--version', action='version', version=__version__)

    parser.add_argument('--user', help=_("File owner will be changed to this parameter. It does nothing if it's not set."), action="store", default=None)
    parser.add_argument('--group', help=_("File owner group will be changed to this parameter. It does nothing if it's not set."), action="store", default=None)
    parser.add_argument('--files', help=_("File permissions to set in all files. It does nothing if it's not set."), default=None, metavar='PERM')
    parser.add_argument('--directories', help=_("Directory permissions to set in all directories. It does nothing if it's not set."), default=None, metavar='PERM')
    parser.add_argument('--remove_emptydirs', help=_("If it's established, removes empty directories recursivily from current path."), action="store_true", default=False)
    parser.add_argument('--only', help=_("Only changes permissions to the file or directory passed in absolute_path parameter."), action="store_true", default=False)
    parser.add_argument('absolute_path', help=_("Directory who is going to be changed permissions and owner recursivily"), action="store")

    args=parser.parse_args(arguments)

    colorama_init(autoreset=True)

    # Sets locale to get integer format localized strings
    try:
        locale.setlocale(locale.LC_ALL, ".".join(locale.getlocale()))
    except:
        pass

    if os.path.isabs(args.absolute_path)==False:
        print(Fore.RED + Style.BRIGHT + _("Path parameter must be an absolute one") + Style.RESET_ALL)
        sys.exit(1)

    if not (is_octal_string_permissions_valid(args.files) and is_octal_string_permissions_valid(args.directories)):
        print(Fore.RED + Style.BRIGHT + _("Seems you gave a bad octal string in --files or --directories parameters. Use format 644 or 755 for example."))
        sys.exit(1)

    deleted_dirs=[]
    files=[]
    dirs=[]
    changed_dirs=[]
    changed_files=[]
    error_files=[]

    #Generate list of files and directories
    if args.only==False:
        dirs.append(args.absolute_path)
        for (dirpath, dirnames, filenames) in os.walk(args.absolute_path):
            for d in dirnames:
                dirs.append(os.path.join(dirpath, d))
            for f in filenames:
                files.append(os.path.join(dirpath, f))
    else:
        if os.path.isdir(args.absolute_path):
            dirs.append(args.absolute_path)
        else:
            files.append(args.absolute_path)

    #Iterate list of dirs
    for dirname in dirs:
        if os.path.exists(dirname)==False:
            error_files.append(dirname)
            continue

        b_permissions=set_octal_string_permissions(dirname,args.directories)
        b_ownership=set_file_ownership(dirname, args.user, args.group)
        if b_permissions==True or b_ownership==True:
            changed_dirs.append(dirname)

        if args.remove_emptydirs==True:
            if is_dir_empty(dirname):
                os.rmdir(dirname)
                deleted_dirs.append(dirname)

    #Iterate list of files
    for filename in files:
        if os.path.exists(filename)==False:
            error_files.append(filename)
            continue

        b_permissions=set_octal_string_permissions(filename, args.files)
        b_ownership=set_file_ownership(filename, args.user, args.group)
        if b_permissions or b_ownership==True:
            changed_files.append(dirname)

    print(Style.BRIGHT + _("RecPermissions summary:"))
    print(Style.BRIGHT + Fore.GREEN + "  * " + Fore.RESET + _("Directories found: ") + Fore.YELLOW + localized_int(len(dirs)))
    print(Style.BRIGHT + Fore.GREEN + "  * " + Fore.RESET + _("Files found: ") + Fore.YELLOW + localized_int(len(files)))
    print(Style.BRIGHT + Fore.GREEN + "  * " + Fore.RESET + _("Directories changed: ") + Fore.YELLOW + localized_int(len(changed_dirs)))
    print(Style.BRIGHT + Fore.GREEN + "  * " + Fore.RESET + _("Files changed: ") + Fore.YELLOW + localized_int(len(changed_files)))
    print(Style.BRIGHT + Fore.GREEN + "  * " + Fore.RESET + _("Directories deleted: ") + Fore.YELLOW + localized_int(len(deleted_dirs)))
    if len(error_files)>0:
        print(Style.BRIGHT + Fore.GREEN + "  * " + Fore.RESET +  _("{} error files:").format(Fore.RED + localized_int(len(error_files))+ Fore.RESET))
        for e in error_files:
            print(Style.BRIGHT + Fore.RED + "     + " + Style.RESET_ALL + e)



from setuptools import setup, Command
import datetime
import gettext
import os
import platform
import site

gettext.install('recpermissions', 'recpermissions/locale')

class Doxygen(Command):
    description = "Create/update doxygen documentation in doc/html"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        print("Creating Doxygen Documentation")
        os.system("""sed -i -e "41d" doc/Doxyfile""")#Delete line 41
        os.system("""sed -i -e "41iPROJECT_NUMBER         = {}" doc/Doxyfile""".format(__version__))#Insert line 41
        os.system("rm -Rf build")
        os.chdir("doc")
        os.system("doxygen Doxyfile")
        os.system("rsync -avzP -e 'ssh -l turulomio' html/ frs.sourceforge.net:/home/users/t/tu/turulomio/userweb/htdocs/doxygen/recpermissions/ --delete-after")
        os.chdir("..")

class Procedure(Command):
    description = "Create/update doxygen documentation in doc/html"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        print(_("New Release:"))
        print(_("  * Change version and date in version.py"))
        print(_("  * Edit Changelog in README"))
        print("  * python setup.py doc")
        print("  * mcedit locale/es.po")
        print("  * python setup.py doc")
        print("  * python setup.py install")
        print("  * python setup.py doxygen")
        print("  * mcedit doc/ttyrec/howto.py")
        print("  * python setup.py video" + ". " + _("If changed restart from first python setup.py doc"))
        print("  * git commit -a -m 'recpermissions-X.X.X'")
        print("  * git push")
        print(_("  * Make a new tag in github"))
        print("  * python setup.py sdist upload -r pypi")
        print(_("  * Create a new gentoo ebuild with the new version"))
        print(_("  * Upload to portage repository")) 

class Uninstall(Command):
    description = "Uninstall installed files with install"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if platform.system()=="Linux":
            os.system("rm -Rf {}/recpermissions*".format(site.getsitepackages()[0]))
            os.system("rm /usr/bin/recpermissions")
            os.system("rm /usr/share/man/man1/recpermissions.1")
            os.system("rm /usr/share/man/es/man1/recpermissions.1")
        else:
            print(_("Uninstall command only works in Linux"))

class Doc(Command):
    description = "Update man pages and translations"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        #es
        os.system("xgettext -L Python --no-wrap --no-location --from-code='UTF-8' -o locale/recpermissions.pot *.py recpermissions/*.py doc/ttyrec/*.py")
        os.system("msgmerge -N --no-wrap -U locale/es.po locale/recpermissions.pot")
        os.system("msgmerge -N --no-wrap -U locale/fr.po locale/recpermissions.pot")
        os.system("msgfmt -cv -o recpermissions/locale/es/LC_MESSAGES/recpermissions.mo locale/es.po")
        os.system("msgfmt -cv -o recpermissions/locale/fr/LC_MESSAGES/recpermissions.mo locale/fr.po")

        for language in ["en", "es", "fr"]:
            self.mangenerator(language)

    def mangenerator(self, language):
        """
            Create man pages for parameter language
        """
        from mangenerator import Man
        if language=="en":
            lang1=gettext.install('recpermissions', 'badlocale')
            man=Man("man/man1/recpermissions")
        else:
            lang1=gettext.translation('recpermissions', 'recpermissions/locale', languages=[language])
            lang1.install()
            man=Man("man/{}/man1/recpermissions".format(language))
        print("  - DESCRIPTION in {} is {}".format(language, _("DESCRIPTION")))

        man.setMetadata("RecPermissions",  1,   datetime.date.today(), "Mariano Muñoz", _("Change files and directories owner and permissions recursively."))
        man.setSynopsis("""usage: recpermissions [-h] [--version] [--user USER] [--group GROUP]
                      [--files PERM] [--directories PERM] [--remove_emptydirs]
                      [--only]
                      absolute_path""")
        man.header(_("DESCRIPTION"), 1)
        man.paragraph(_("This app has the following mandatory parameters:"), 1)
        man.paragraph("--user", 2, True)
        man.paragraph(_("User used to change files owner.") + " " + _("It does nothing if it's not set"), 3)
        man.paragraph("--group", 2, True)
        man.paragraph(_("Group used to change files owner group.") + " " + _("It does nothing if it's not set"), 3)
        man.paragraph("--files", 2, True)
        man.paragraph(_("File permissions in octal to be used.") + " " + _("It does nothing if it's not set"), 3)
        man.paragraph("--directories", 2, True)
        man.paragraph(_("Directory permissions in octal to be used.") + " " + _("It does nothing if it's not set"), 3)
        man.paragraph("--remove_emptydirs", 2, True)
        man.paragraph(_("When used in script, removes all empty dirs recursively from path."), 3)
        man.paragraph("--only", 2, True)
        man.paragraph(_("Only changes permissions to the file or directory passed in absolute_path parameter."), 3)
        man.paragraph("absolute_path", 2, True)
        man.paragraph(_("To avoid errors and wrong changes, path must be an absolute one."), 3)  
        man.header(_("EXAMPLES"), 1)
        man.paragraph(_("Null Example"), 2, True)
        man.paragraph(_("recpermissions /home/user/"), 3)
        man.paragraph(_("This comand does nothing"), 3)
        man.paragraph(_("Partial Example"), 2, True)
        man.paragraph(_("recpermissions --user user --files 644 /home/user/"), 3)
        man.paragraph(_("This command only changes user permissions and files permissions to 644. Group and directory permissions are not changed:"), 3)
        man.paragraph(_("Full Example"), 2, True)
        man.paragraph(_("recpermissions --user root --group root --files 640 --directories 750 --remove_emptydirs /home/user/"), 3)
        man.paragraph(_("This command will change user and group to root user and group. Files will have rw-r----- permisions and directories rwxr-x--- permisions. If the script finds empty dirs it will remove them:"), 3)
        man.paragraph(_("Only Example"), 2, True)
        man.paragraph(_("recpermissions --user user /home/user/README.txt --only"), 3)
        man.paragraph(_("This command will change user ownership to the file /home/usr/README.txt only"),  3)
        man.save()

class Video(Command):
    description = "Create video/GIF from console ouput"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        print(_("You need ttyrecgenerator installed to generate videos"))
        os.chdir("doc/ttyrec")
        os.system("ttyrecgenerator --output recpermissions_howto_es 'python3 howto.py' --lc_all es_ES.UTF-8")
        os.system("ttyrecgenerator --output recpermissions_howto_en 'python3 howto.py' --lc_all C")
        os.chdir("../..")

    ########################################################################


## Version of modele captured from version to avoid problems with package dependencies
__version__= None
with open('recpermissions/version.py', encoding='utf-8') as f:
    for line in f.readlines():
        if line.find("__version__ =")!=-1:
            __version__=line.split("'")[1]


with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

if platform.system()=="Linux":
    data_files=[('/usr/share/man/man1/', ['man/man1/recpermissions.1']), 
                ('/usr/share/man/es/man1/', ['man/es/man1/recpermissions.1'])
               ]
else:
    data_files=[]

setup(name='recpermissions',
    version=__version__,
    description='Change files and directories permisions and owner recursivily from current directory',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=['Development Status :: 4 - Beta',
                 'Intended Audience :: System Administrators',
                 'Topic :: System :: Systems Administration',
                 'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
                 'Programming Language :: Python :: 3',
                ],
    keywords='change permissions ownner files directories',
    url='https://github.com/Turulomio/recpermissions',
    author='Turulomio',
    author_email='turulomio@yahoo.es',
    license='GPL-3',
    packages=['recpermissions'],
    entry_points = {'console_scripts': ['recpermissions=recpermissions.core:main',
                                       ],
                   },
    install_requires=['colorama','setuptools'],
    data_files=data_files,
    cmdclass={ 'doxygen': Doxygen,
               'doc': Doc,
               'uninstall': Uninstall,
               'video': Video,
               'procedure': Procedure,
             },
    zip_safe=False,
    include_package_data=True
    )

_=gettext.gettext#To avoid warnings
